package it.unibo.tw0708.spring.scopes;

import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.ObjectFactory;
import org.springframework.beans.factory.config.Scope;

public class NonHttpBasedSessionScope 
implements Scope {

	private static Map<ClassLoader,Map<String,Object>> map = new HashMap<ClassLoader,Map<String,Object>>();

	// -------------------------------------------------------------
	
    /**
     * Returns the object with the given name from the underlying scope, creating it if not found in the underlying storage mechanism.
     */
    public Object get(String beanName, ObjectFactory factory) {
    	ClassLoader currentClassLoader = Thread.currentThread().getContextClassLoader();
    	Map<String,Object> currentClassLoaderMap = map.get(currentClassLoader);
    	if ( currentClassLoaderMap == null ) {
    		currentClassLoaderMap = new HashMap<String,Object>();
    		map.put(currentClassLoader,currentClassLoaderMap);
    	}
    	Object bean = currentClassLoaderMap.get(beanName);
    	if ( bean == null ) {
    		bean = factory.getObject();
    		currentClassLoaderMap.put(beanName,bean);
    	}
		return bean;
	}

    /**
     * Remove the object with the given name from the underlying scope.
     */
	public Object remove(String beanName) {
    	ClassLoader currentClassLoader = Thread.currentThread().getContextClassLoader();
    	Map<String,Object> currentClassLoaderMap = map.get(currentClassLoader);
    	if ( currentClassLoaderMap == null ) {
    		return null;
    	}
    	return currentClassLoaderMap.remove(beanName);
	}

    /**
     * Registers a callback to be executed on destruction of the specified object in the scope 
     * (or at destruction of the entire scope, if the scope does not destroy individual objects 
     * but rather only terminates in its entirety).
     */
	public void registerDestructionCallback(String arg0, Runnable arg1) {
		// I don't mind here
	}

	/**
	 * Returns the conversation id for the current underlying scope, if any.
	 */
	public String getConversationId() {
    	ClassLoader currentClassLoader = Thread.currentThread().getContextClassLoader();
		return currentClassLoader.toString();
	}

}
