package it.unibo.tw0708.dao.tx.jdbc;

import it.unibo.tw0708.dao.CompactDiscDAO;
import it.unibo.tw0708.dao.CompactDiscTO;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

public class JdbcTemplateTxCompactDiscDAO 
implements CompactDiscDAO {

	private JdbcTemplate jdbcTemplate;
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}
	
	private static final String TABLE = "compactDiscs";

	private static final String ID = "id";
	private static final String OWNER = "owner";
	private static final String AUTHOR = "author";
	private static final String TITLE = "title";
	
	// ----------------------------------------------------------------
	
	public List<CompactDiscTO> getCompactDiscs(String username) {
		Collection results = jdbcTemplate.query(
			"select * from "+ TABLE + " where " + OWNER + " = ?",
			new Object[] { username },
			new CompactDiscRowMapper()
		);
		List<CompactDiscTO> compactDiscs = new ArrayList<CompactDiscTO>();
		for ( Object result : results ) {
			compactDiscs.add( (CompactDiscTO) result );
		}
		return compactDiscs;
	}

	public void addCompactDiscs(CompactDiscTO compactDisc) {
		jdbcTemplate.update(
			"insert into " + TABLE + " (" + OWNER + ", " + AUTHOR + ", " + TITLE + ") values (?, ?, ?)", 
			new Object[] { compactDisc.getOwner(), compactDisc.getAuthor(), compactDisc.getTitle() }
		);
	}

	public void removeCompactDiscs(CompactDiscTO compactDisc) {
		jdbcTemplate.update(
				"delete from " + TABLE + " where " + ID + " = ?", 
				new Object[] { new Integer(compactDisc.getId()) }
			);
	}

	// ----------------------------------------------------------------

	public void createTable() {
		// this only works with MySQL !!!
//		jdbcTemplate.execute("create table if not exists " + TABLE + " (" + ID + " integer, " + OWNER + " varchar(100), " + AUTHOR + " varchar(100), " + TITLE + " varchar(100))");
		try { jdbcTemplate.execute("create table " + TABLE + " (" + ID + " integer not null identity, " + OWNER + " varchar(100), " + AUTHOR + " varchar(100), " + TITLE + " varchar(100), unique ( " + ID + " ) )"); }
		catch (Exception e) {
			// TODO: handle exception
		}
	}
		
	// -----------------------------------------------------------------
		
	private class CompactDiscRowMapper 
	implements RowMapper {
	    
		public Object mapRow(ResultSet rs, int rowNum) 
	    throws SQLException {
	        CompactDiscTO compactDisc = new CompactDiscTO();
	        compactDisc.setId(rs.getInt(ID));
	        compactDisc.setOwner(rs.getString(OWNER));
	        compactDisc.setAuthor(rs.getString(AUTHOR));
	        compactDisc.setTitle(rs.getString(TITLE));
	        return compactDisc;
	    }
		
	}

}
