package it.unibo.tw0708.junit;

import it.unibo.tw0708.beans.compactDiscs.ICompactDiscsManager;
import it.unibo.tw0708.beans.user.UserManager;

import org.springframework.test.AbstractDependencyInjectionSpringContextTests;

public class MySpringTestCase
extends AbstractDependencyInjectionSpringContextTests {

	/**
	 * Specifies the Spring configuration to load for this test fixture
	 */
	protected String[] getConfigLocations() {
	    return new String[] { "classpath:*junitApplicationContext.xml" };
	}

	// -----------------------------------------------------------------------------------------
	
	public void testExplicitBeanLookup() {
		UserManager userManager = (UserManager) applicationContext.getBean("userManager");
		assertNotNull("userManager lookup",userManager);
		ICompactDiscsManager compactDiscsManager = (ICompactDiscsManager) applicationContext.getBean("compactDiscsManager");
		assertNotNull("compactDiscsManager lookup",compactDiscsManager);
	}
	
	private ICompactDiscsManager compactDiscsManager;
	public void setCompactDiscsManager(ICompactDiscsManager compactDiscsManager) { this.compactDiscsManager = compactDiscsManager; }

	private UserManager userManager;
	public void setUserManager(UserManager userManager) { this.userManager = userManager; }

	public void testJUnitTestCaseAutowiring() {
		assertNotNull("userManager test case autowiring",userManager);
		assertNotNull("compactDiscsManager test case autowiring",compactDiscsManager);
	}

	public void testDependencyInjection() {
		UserManager userManager = compactDiscsManager.getOwner();
		assertNotNull("userManager dependency injection",userManager);
	}

	public void testJDKInterfaceProxyPrototypeBehaviour() {
		compactDiscsManager.setSomeProperty("aValue");					
		assertNull("jdk interface proxied prototype behaves completely stateless",compactDiscsManager.getSomeProperty());
	}
	
	public void testSessionScopedBehaviour() {
		String aUser = "aUser";
		userManager.setName(aUser);					
		assertEquals("session scoped behaviour retains conversation values",userManager.getName(),aUser);
	}

	public void testFakeDaoImplementationConstructorAndFieldInjection() {
		String username = "pisi";
		int expectedCollectionSize = 2;
		compactDiscsManager.getOwner().setName(username);
		int actualCollectionSize = compactDiscsManager.getCollectionSize();
		assertEquals("user " + username + " owns the correct number of compact discs",expectedCollectionSize,actualCollectionSize);
	}

}
